#include <sys/types.h>
#include <time.h>

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <inttypes.h>

#include "monomial.h"

int main(int argc, char *argv[])
{
    monomial_t m, m1, m2;
    monomial_t * list;
    uint32_t len;
    uint32_t i;
    
    char * out;
    char * out1;
    char * out2;
    
    ///////////////////////////////////////////////////////////////////////////
    // INIT, FROM_STRING, CLEAR
    
    MON_INIT(m);
    m = mon_from_string("3  1 1 1");
    
    out = mon_to_string(m, 3);
    printf("m = \"%s\"\n", out);
    free(out);
    
    MON_CLEAR(m);
    
    ///////////////////////////////////////////////////////////////////////////
    // GENERATE_BY_DEGREE

    printf("Generating all monomials in X, Y, Z of degree 6\n");
    list = mon_generate_by_degree(&len, 2, 6);
    for (i = 0; i < len; i++)
    {
        out = mon_to_string_pretty(list[i], 2, NULL);
        printf("%s\n", out);
        free(out);
    }
    for (i = 0; i < len; i++)
        MON_CLEAR(list[i]);
    free(list);
    
    
    printf("Generating all monomials in X, Y, Z of degree 6\n");
    list = mon_generate_by_degree_invlex(&len, 2, 6);
    for (i = 0; i < len; i++)
    {
        out = mon_to_string_pretty(list[i], 2, NULL);
        printf("%" PRIu64 " - %s\n", list[i], out);
        free(out);
    }
    for (i = 0; i < len; i++)
        MON_CLEAR(list[i]);
    free(list);
    
    ///////////////////////////////////////////////////////////////////////////
    // IS_ONE, DEGREE

    MON_INIT(m);
    m = mon_from_string("3  2 0 1");
    out = mon_to_string_pretty(m, 2, "XYZ");
    printf("Is %s equal to one?  %d\n", out, MON_IS_ONE(m));
    printf("Degree: %d\n", mon_degree(m));
    free(out);
    MON_CLEAR(m);
    
    MON_INIT(m);
    m = mon_from_string("3  0 0 0");
    out = mon_to_string_pretty(m, 2, "XYZ");
    printf("Is %s equal to one?  %d\n", out, MON_IS_ONE(m));
    printf("Degree: %d\n", mon_degree(m));
    free(out);
    MON_CLEAR(m);
    
    ///////////////////////////////////////////////////////////////////////////
    // MUL, DIVIDES, INVLEX
    
    MON_INIT(m);
    MON_INIT(m1);
    MON_INIT(m2);
    m1 = mon_from_string("3  2 0 1");
    m2 = mon_from_string("3  0 1 0");
    MON_MUL(m, m1, m2);
    out = mon_to_string_pretty(m, 2, "XYZ");
    printf("Product is [2 0 1] [0 1 0] = %s\n", out);
    free(out);
    printf("Divides?  %d\n", mon_divides(m1, m2));
    printf("INVLEX: %d\n", MON_CMP_INVLEX(m1, m2));
    MON_CLEAR(m1);
    MON_CLEAR(m2);
    MON_CLEAR(m);
    
    ///////////////////////////////////////////////////////////////////////////
    // DIVIDES, DIV, INVLEX
    
    MON_INIT(m);
    MON_INIT(m1);
    MON_INIT(m2);
    m1 = mon_from_string("3  1 0 1");
    m2 = mon_from_string("3  1 1 3");
    MON_MUL(m, m1, m2);
    out = mon_to_string_pretty(m, 2, "XYZ");
    printf("Product is [1 0 1] [1 1 3] = %s\n", out);
    free(out);
    printf("Divides?  %d\n", mon_divides(m1, m2));
    MON_DIV(m, m2, m1);
    out = mon_to_string_pretty(m, 2, "XYZ");
    printf("Quotient: %s\n", out);
    free(out);
    printf("INVLEX: %d\n", MON_CMP_INVLEX(m1, m2));
    MON_CLEAR(m1);
    MON_CLEAR(m2);
    MON_CLEAR(m);
    
    ///////////////////////////////////////////////////////////////////////////
    // INVLEX
    
    m1 = mon_from_string("3  6 0 0");
    m2 = mon_from_string("3  0 0 6");
    out1 = mon_to_string_pretty(m1, 2, NULL);
    out2 = mon_to_string_pretty(m2, 2, NULL);
    printf("CMP_INVLEX(%s, %s) = %d\n", out1, out2, MON_CMP_INVLEX(m1, m2));
    free(out1);
    free(out2);
}

